#include <ESP8266WiFi.h>
#include <BlynkSimpleEsp8266.h>
#include <Adafruit_NeoPixel.h>
#include <NTPClient.h>
#include <WiFiUdp.h>
#include <Adafruit_Sensor.h>
#include <DHT.h>
#include <DHT_U.h>
#include <Adafruit_MQTT.h>
#include <Adafruit_MQTT_Client.h>
#include "AdafruitIO_WiFi.h"

// MQTT Data
#define MQTT_SERVER      "io.adafruit.com"
#define MQTT_SERVERPORT  1883
#define MQTT_USERNAME    "<adafruit_io_username>"
#define MQTT_KEY         "<adafruit_io_key>"

// Wifi Data
#define WIFI_SSID 		"<wifi_ssid>"
#define WIFI_PASSPHRASE "<wifi_password>"

// Pins Data
#define PIEZO_PIN 		A0
#define LED_MATRIX_PIN 	D1
#define TRIGGER_PIN 	D2
#define ECHO_PIN 		D3
#define DHTPIN 			D4
#define REDPIN 			D5
#define GREENPIN 		D6
#define BLUEPIN 		D7
#define SOUNDPIN 		D8

// Colors Constants
#define COLOR_PURPLE 	strip.Color(100, 0, 100)
#define COLOR_RED 		strip.Color(100, 0, 0)

#define COLOR_CURRENT_TIME 	COLOR_PURPLE
#define COLOR_TEMPERATURE 	COLOR_PURPLE
#define COLOR_EXPRESSION 	COLOR_PURPLE
#define COLOR_ALARM_SET 	COLOR_RED

// Display Constants
#define DISPLAY_TIME_MIN_DISPLAY 		-17
#define TIMEZONE 						7200
#define UPDATE_TIME_DELAY 				1000

#define NUMERIC_EXPRESSION_MIN_DISPLAY 	-16
#define SOLUTION_BOUNDRY 				100

#define MAX_PHASE 						8

#define DISPLAY_TIME_MIN_TEMPERATURE 	-9
#define TIME_TO_SHOW_TEMPERATURE 		10000

// Constants for Distance Detection
#define MAX_DISTANCE 		100
#define ACTIVATED_DISTANCE 	10

// Number Constants
#define PLUS 	10
#define MINUS 	11
#define EQUALS 	12
#define POINTED 13

// Piazo Constants
#define THRESHOLD 20

// DHT Constants
#define DHTTYPE DHT22

// Other Consts
#define ALARM_SPEED 70
#define DRAW_ROW 	1

// Musical notes
const float octav[2][7] = {	{220, 246.94, 261.625, 293.665, 329.625, 349.23, 391.995},
							{440, 493.88, 523.25, 587.33, 659.25, 698.46, 783.99}};

#define DO 	0
#define RE 	1
#define MI 	2
#define FA 	3
#define SOL 4
#define LA 	5
#define SI 	6

const int ALADIN_NOTES[] = {SI, DO, MI, RE, SI, DO, MI, RE, LA, DO, SI, SI, SI, DO, RE, FA, LA, SOL, FA, SOL, MI, FA, LA, SOL, SI};
const int ALADIN_OCTAV[] = {0,  1,  1,  1,  0,  1,  1,  1,  0,  1,  0,  0,  0,  1,  1,  0,  0,  0,   0,  0,   0,  0,  0,  0,   0};

// State Machine
enum MAIN_STATES {
  ALARM_OFF, ALARM_ON
};

enum STATES {
  SHOW_CURRENT_TIME, NEW_ALARM_SET, SHOW_ALARM_SET, 												     // Alarm OFF states
  ALARM_WAITING_FOR_APPROCH, ALARM_WAITING_FOR_KNOCKING, ALARM_SOLVE_EQUATION, ALARM_SHOWING_TEMPERATURE // Alarm ON states
};

// Feed Constants
const char WAKE_UP_TIME_FEED[] 	= MQTT_USERNAME "/feeds/WakeupTime";
const char FAST_REACTION_FEED[] = MQTT_USERNAME "/feeds/FastReaction";
const char TEMPERATURE_FEED[] 	= MQTT_USERNAME "/feeds/Temperature";

// Blynk Constants
const char auth[] = "mcdqarpWO7tRvkWElLna4Nm9e28ZHWFx";

WiFiUDP ntpUDP;
WiFiClient client;

DHT_Unified dht(DHTPIN, DHTTYPE);
NTPClient timeClient(ntpUDP, "pool.ntp.org", TIMEZONE);
Adafruit_NeoPixel strip = Adafruit_NeoPixel(64, LED_MATRIX_PIN, NEO_GRB + NEO_KHZ800);
Adafruit_MQTT_Client mqtt(&client, MQTT_SERVER, MQTT_SERVERPORT, MQTT_USERNAME, MQTT_KEY);
Adafruit_MQTT_Publish temperatureFeed = Adafruit_MQTT_Publish(&mqtt, TEMPERATURE_FEED);
Adafruit_MQTT_Publish wakeUpTimeFeed = Adafruit_MQTT_Publish(&mqtt, WAKE_UP_TIME_FEED);

MAIN_STATES mainState;
STATES secondaryState;
int currentPhase, fastAmount;
int timePassedSinceLastTimeUpdate = 0;
boolean finishShowingTime = false, sentTemperature = false;

// Alarm Parameters
long timeToWakeUp, timePassedSinceShowingTemp;
int alarmHours, alarmMinutes;

// Equations Parameters
int firstNumber, secondNumber, action, solution, userSolution = SOLUTION_BOUNDRY;

// Piazo Parameters
int knockCounter = 0;

// Alarm Light & Sound Parameters
long alarmTime;
int currentLightStage, currentSoundStage;

void setup() {
  Serial.begin(9600);
  WiFi.begin(WIFI_SSID, WIFI_PASSPHRASE);

  Serial.print("Connecting to WIFI");
  Serial.println();
  
  while(WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  
  Serial.println("Connected");
  timeClient.begin();
  strip.begin();
  dht.begin();

  timeClient.update(); 
  timePassedSinceLastTimeUpdate = millis();
  
  strip.clear();
  strip.show();
  Blynk.begin(auth, WIFI_SSID, WIFI_PASSPHRASE);
  mqttConnect();

  mainState = ALARM_OFF;
  secondaryState = SHOW_CURRENT_TIME;
  currentPhase = MAX_PHASE;

  fastAmount = 0;
  Blynk.virtualWrite(V2, fastAmount);

  Serial.println("Connected");

  // Stage 1 - for when the alarm is only showing the clock
//  pinMode(REDPIN, OUTPUT);

  // Stage 2 - for when you want to activate the alarm
//  alarmMinutes = 49;
//  alarmHours = 15;
//  pinMode(REDPIN, OUTPUT);
//  pinMode(GREENPIN, OUTPUT);
//  pinMode(BLUEPIN, OUTPUT);
//  pinMode(SOUNDPIN, OUTPUT);

  // Stage 3 - for when the alarm is activate and you want to turn it off by aprroaching and knocking
//  mainState = ALARM_ON;
//  secondaryState = ALARM_WAITING_FOR_APPROCH;
//  Serial.print("Waiting for approch to deactivate... ");

  // Stage 4 - for when the alarm is active and the only thing that left is to answer the equation
//  mainState = ALARM_ON;
//  secondaryState = ALARM_SOLVE_EQUATION;
//  generateEquation();
//  timeToWakeUp = millis();
//  timePassedSinceShowingTemp = millis();
//  alarmTime = millis();
}

void mqttConnect() {
  Serial.print("Connecting to Adafruit IO... ");
  int8_t ret;
  while ((ret = mqtt.connect()) != 0) {
    Serial.print(mqtt.connectErrorString(ret));

    if (ret >= 0) {
      mqtt.disconnect();
    }
    
    Serial.println("Retrying To Connect... ");
    delay(5000);
  }
  Serial.println("Adafruit IO Connected!");
}

void loop() {
  Blynk.run();
  
  retriveCurrentTime();
  checkAlarm();

  if(isAlarmActivate()) {
    handleAlarmActivate();
	
	if(secondaryState != ALARM_SHOWING_TEMPERATURE) {
		activeLed();
		activeSound();
		alarmTime = millis();
	}
  } else {
    handleAlarmNotActivate();
  }

  strip.show();
  delay(300);
  strip.clear();
}

void retriveCurrentTime() {
  if(millis() - timePassedSinceLastTimeUpdate >= UPDATE_TIME_DELAY) {    
    timeClient.update();
    timePassedSinceLastTimeUpdate = millis();
  }
}

void activeSound() {
  if(millis() - alarmTime < ALARM_SPEED) {
    return;
  }

  tone(SOUNDPIN, octav[ALADIN_OCTAV[currentSoundStage]][ALADIN_NOTES[currentSoundStage]], 100);
  currentSoundStage++;

  if(currentSoundStage == 25) {
    currentSoundStage = 0;
  }
}

void activeLed() {
  if(millis() - alarmTime < ALARM_SPEED) {
    return;
  }

  switch(currentLightStage) {
    case 0:
      digitalWrite(REDPIN, LOW);
      currentLightStage++;
      break;
    case 1:
      digitalWrite(REDPIN, HIGH);
      currentLightStage++;
      break;
    case 2:
      digitalWrite(GREENPIN, LOW);
      currentLightStage++;
      break;
    case 3:
      digitalWrite(GREENPIN, HIGH);
      currentLightStage++;
      break;
    case 4:
      digitalWrite(BLUEPIN, LOW);
      currentLightStage++;
      break;
    case 5:
      digitalWrite(BLUEPIN, HIGH);
      currentLightStage = 0;
      break;
  }
}

void checkAlarm() {
  if(isAlarmActivate() || timeClient.getHours() != alarmHours || timeClient.getMinutes() != alarmMinutes) {
    return;
  }

  mainState = ALARM_ON;
  currentPhase = MAX_PHASE;
  generateEquation();
  currentLightStage = 0;
  alarmTime = millis();
  secondaryState = ALARM_WAITING_FOR_APPROCH;
  updateLedPins();
  timeToWakeUp = millis();
  Serial.print("Waiting for approch to deactivate... ");
}

void updateLedPins() {
  digitalWrite(REDPIN, HIGH);
  digitalWrite(GREENPIN, HIGH);
  digitalWrite(BLUEPIN, HIGH);
}

void handleAlarmActivate() {
  int cm;
  
  switch(secondaryState) {
    case ALARM_WAITING_FOR_KNOCKING:
      if(isKnock()) {
        switch (knockCounter) {
          case 0:
            Serial.println("First Knock");
            knockCounter++;
            break;
          case 1:
            Serial.println("Second Knock");
            knockCounter++;
            break;        
          case 2:
            currentPhase = ALARM_SOLVE_EQUATION;
            knockCounter = 0;
            Serial.println("Third Knock");
            break;
        }
        debounce();
      }
      break;
    case ALARM_WAITING_FOR_APPROCH:
      cm = ultrasonicMeasure(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); 

      if(cm <= ACTIVATED_DISTANCE && cm > 1) {
        secondaryState = ALARM_WAITING_FOR_KNOCKING;
        Serial.println("Movement Detected... Waiting for knocking");
      }
      break;
    case ALARM_SOLVE_EQUATION:
      if(solution == userSolution) {
        secondaryState = ALARM_SHOWING_TEMPERATURE;
        userSolution = SOLUTION_BOUNDRY;
        timePassedSinceShowingTemp = millis();
        wakeUpTimeFeed.publish(((int) (millis() - timeToWakeUp) / 1000));
      }
      
      drawNumericExpression(COLOR_EXPRESSION, firstNumber, secondNumber, action);
      break;
    case ALARM_SHOWING_TEMPERATURE:
      if(millis() - timePassedSinceShowingTemp > TIME_TO_SHOW_TEMPERATURE)
      {
        mainState = ALARM_OFF;
        secondaryState = SHOW_CURRENT_TIME;
        currentPhase = MAX_PHASE;
        sentTemperature = false;
      }
      
      sensors_event_t event;  
      dht.temperature().getEvent(&event);
      drawTemperature(COLOR_TEMPERATURE, event.temperature);

      if(!sentTemperature) {
        temperatureFeed.publish(event.temperature);
        sentTemperature = true;
      }
      
      break;
  }
}

void handleAlarmNotActivate() {
  switch(secondaryState) {
    case SHOW_CURRENT_TIME:
      drawTime(COLOR_CURRENT_TIME, timeClient.getHours(), timeClient.getMinutes());
      break;
    case NEW_ALARM_SET:
      currentPhase = MAX_PHASE;
      secondaryState = SHOW_ALARM_SET;

      drawTime(COLOR_ALARM_SET, alarmHours, alarmMinutes);
      break;
    case SHOW_ALARM_SET:
      if(!finishShowingTime) {
        drawTime(COLOR_ALARM_SET, alarmHours, alarmMinutes);
      } else {
        secondaryState = SHOW_CURRENT_TIME;
      }
      break;
  }
}

boolean isKnock() {
  int val = analogRead(PIEZO_PIN);
  if(val > THRESHOLD) {
    return true;
  } else {
    return false;
  }
}

void debounce() {
  int debouncer = 0;
  while(debouncer < 10) {
    int val = analogRead(PIEZO_PIN);
    if (val < THRESHOLD) {
      debouncer++;
    }
    else {
      debouncer = 0;
    }
    delay(5);
  }
}

void generateEquation() {
  firstNumber = random(10);
  secondNumber = random(10);
  action = random(2) + PLUS;

  if(action == PLUS) {
    solution = firstNumber + secondNumber;
  } else {
    solution = firstNumber - secondNumber;
  }
}

boolean isAlarmActivate() {
  return mainState == ALARM_ON;
}

void drawTemperature(uint32_t color, int temperature) {
  drawByNum(color, DRAW_ROW, 0, temperature / 10);  
  drawByNum(color, DRAW_ROW, 4, temperature % 10);
  strip.setPixelColor(7, color);
}

void drawNumericExpression(uint32_t color, int firstNumber, int secondNumber, int action) {
  if(finishShowingTime) {
    finishShowingTime = false;
  }
  
  drawByNum(color, DRAW_ROW, currentPhase, firstNumber);  
  drawByNum(color, DRAW_ROW, currentPhase + 4, action);
  drawByNum(color, DRAW_ROW, currentPhase + 8, secondNumber);
  drawByNum(color, DRAW_ROW, currentPhase + 12, EQUALS);

  currentPhase--;

  if(currentPhase == NUMERIC_EXPRESSION_MIN_DISPLAY) {
    currentPhase = MAX_PHASE;
    finishShowingTime = true;
  }
}

void drawTime(uint32_t color, int hour, int minutes) {
  if(finishShowingTime) {
    finishShowingTime = false;
  }
  
  drawByNum(color, DRAW_ROW, currentPhase, hour / 10);  
  drawByNum(color, DRAW_ROW, currentPhase + 4, hour % 10);

  drawByNum(color, DRAW_ROW, currentPhase + 7, POINTED);

  drawByNum(color, DRAW_ROW, currentPhase + 10, minutes / 10);
  drawByNum(color, DRAW_ROW, currentPhase + 14, minutes % 10);

  currentPhase--;

  if(currentPhase == DISPLAY_TIME_MIN_DISPLAY) {
    currentPhase = MAX_PHASE;
    finishShowingTime = true;
  }
}

int ultrasonicMeasure(int trigPin, int echoPin, int max_distance) {
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(trigPin, LOW);
  int duration = pulseIn(echoPin, HIGH, max_distance * 59);
  return duration / 59;
}

BLYNK_WRITE(V0) {
  int timeInSecsFromBlynk = param[0].asLong();

  alarmHours = timeInSecsFromBlynk / 3600;
  alarmMinutes = (timeInSecsFromBlynk - alarmHours * 3600) / 60;

  if(!isAlarmActivate()) {
    secondaryState = NEW_ALARM_SET;
    digitalWrite(REDPIN, HIGH);
  }
}

BLYNK_WRITE(V1) {
  userSolution = param[0].asInt();
}

void drawByNum(uint32_t color, int row, int col, int number) {
  int loc = row * 8 + col;
  
  switch (number) {
    case 0: 
      drawZero(color, row, col);
      break;
    case 1:
      drawOne(color, row, col);
      break;
    case 2:
      drawTwo(color, row, col);
      break;
    case 3:
      drawThree(color, row, col);
      break;
    case 4: 
      drawFour(color, row, col);
      break;
    case 5:
      drawFive(color, row, col);
      break;
    case 6:
      drawSix(color, row, col);
      break;
    case 7:
      drawSeven(color, row, col);
      break;
    case 8:
      drawEight(color, row, col);
      break;
    case 9:
      drawNine(color, row, col);
      break;
    case 10:
      drawPlus(color, row , col);
      break;
    case 11:
      drawMinus(color, row , col);
      break;
    case 12:
      drawEqual(color, row , col);
      break;
    case 13:
      drawPointed(color, row , col);
      break;
  }
}

void drawZero(uint32_t color, int row, int col) {
  /*
   * ---
   * - -
   * - -
   * - -
   * ---
   */
  int loc = row * 8 + col;
  
  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 8, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 24, color);
    strip.setPixelColor(loc + 32, color); 
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 33, color);
  }

  if(col > -3 && col < 6) { 
    strip.setPixelColor(loc + 2, color);
    strip.setPixelColor(loc + 10, color);
    strip.setPixelColor(loc + 18, color);
    strip.setPixelColor(loc + 26, color); 
    strip.setPixelColor(loc + 34, color);
  }
}

void drawOne(uint32_t color, int row, int col) {
  /*
   *   -
   *  -- 
   *   -
   *   -
   *   -
   */

 int loc = row * 8 + col;
  
  if(col > -2 && col < 7) { 
    strip.setPixelColor(loc + 9, color);
  }

  if(col > -3 && col < 6) { 
    strip.setPixelColor(loc + 2, color);
    strip.setPixelColor(loc + 10, color);
    strip.setPixelColor(loc + 18, color);
    strip.setPixelColor(loc + 26, color); 
    strip.setPixelColor(loc + 34, color);
  }
}

void drawTwo(uint32_t color, int row, int col) {
  /*
   * ---
   *   -
   * ---
   * -   
   * ---
   */

  int loc = row * 8 + col;
  
  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 24, color);
    strip.setPixelColor(loc + 32, color); 
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 33, color);
    strip.setPixelColor(loc + 17, color);

  }

  if(col > -3 && col < 6) { 
    strip.setPixelColor(loc + 2, color);
    strip.setPixelColor(loc + 10, color);
    strip.setPixelColor(loc + 18, color);
    strip.setPixelColor(loc + 34, color);
  }
}

void drawThree(uint32_t color, int row, int col) {
  /*
   * ---
   *   -
   * ---
   *   -   
   * ---
   */
 int loc = row * 8 + col;
  
  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 32, color);
     
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 17, color);
    strip.setPixelColor(loc + 33, color);
  }

  if(col > -3 && col < 6) { 

    strip.setPixelColor(loc + 2, color);
    strip.setPixelColor(loc + 10, color);
    strip.setPixelColor(loc + 18, color);
    strip.setPixelColor(loc + 26, color); 
    strip.setPixelColor(loc + 34, color);
  }
}
void drawFour(uint32_t color, int row, int col) {
  /*
   *- -
   *- -
   *---
   *  -   
   *  -
   */
  int loc = row * 8 + col;
  
  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
       strip.setPixelColor(loc + 8, color);

    strip.setPixelColor(loc + 16, color);
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 17, color);
  }

  if(col > -3 && col < 6) { 
    strip.setPixelColor(loc + 2, color);
    strip.setPixelColor(loc + 10, color);
    strip.setPixelColor(loc + 18, color);
    strip.setPixelColor(loc + 26, color);
    strip.setPixelColor(loc + 34, color);

  }
}
void drawFive(uint32_t color, int row, int col) {
  /*
   *---
   *-   
   *---
   *  -   
   *---
   */

 int loc = row * 8 + col;
  
  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 8, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 32, color); 
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 17, color);
    strip.setPixelColor(loc + 33, color);
  }

  if(col > -3 && col < 6) { 
    strip.setPixelColor(loc + 2, color);
   strip.setPixelColor(loc + 18, color);
   strip.setPixelColor(loc + 26, color);
   strip.setPixelColor(loc + 34, color);
  }
}

void drawSix(uint32_t color, int row, int col){
  /*
   *---
   *-   
   *---
   *- -   
   *---
   */

 int loc = row * 8 + col;

  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 8, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 24, color);
    strip.setPixelColor(loc + 32, color); 
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 33, color);
    strip.setPixelColor(loc + 17, color);

  }

  if(col > -3 && col < 6) { 
   strip.setPixelColor(loc + 2, color);
   strip.setPixelColor(loc + 18, color);
   strip.setPixelColor(loc + 26, color);
   strip.setPixelColor(loc + 34, color);
  }
}
void drawSeven(uint32_t color, int row, int col){
  /*
   *---
   * - 
   *---
   * -   
   * -
   */
  
int loc = row * 8 + col;

  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 16, color);
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 9, color);
    strip.setPixelColor(loc + 33, color);
    strip.setPixelColor(loc + 17, color);
    strip.setPixelColor(loc + 25, color);

  }

  if(col > -3 && col < 6) { 
   strip.setPixelColor(loc + 18, color);
 
  }
}
void drawEight(uint32_t color, int row, int col) {
  /*
   *---
   *- - 
   *---
   *- -   
   *---
   */

int loc = row * 8 + col;

  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 8, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 24, color);
    strip.setPixelColor(loc + 32, color);
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 33, color);
    strip.setPixelColor(loc + 17, color);

  }

  if(col > -3 && col < 6) { 
   strip.setPixelColor(loc + 2, color);
   strip.setPixelColor(loc + 10, color);
   strip.setPixelColor(loc + 18, color);
   strip.setPixelColor(loc + 26, color);
   strip.setPixelColor(loc + 34, color); 
  }
}
void drawNine(uint32_t color, int row, int col){
  /*
   *---
   *- - 
   *---
   *- -   
   *---
   */
int loc = row * 8 + col;

  if(col > -1 && col < 8) {
    strip.setPixelColor(loc, color);
    strip.setPixelColor(loc + 8, color);
    strip.setPixelColor(loc + 16, color);
    strip.setPixelColor(loc + 32, color);
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 1, color);
    strip.setPixelColor(loc + 33, color);
    strip.setPixelColor(loc + 17, color);

  }

  if(col > -3 && col < 6) { 
   strip.setPixelColor(loc + 2, color);
   strip.setPixelColor(loc + 10, color);
   strip.setPixelColor(loc + 18, color);
   strip.setPixelColor(loc + 26, color);
   strip.setPixelColor(loc + 34, color);
  }
}

void drawEqual(uint32_t color, int row, int col) {
  /*
   *--- 
   *
   *---   
   *
   */
   int loc = row * 8 + col;
   
  if(col > -1 && col < 8) {
    strip.setPixelColor(loc +8, color);
    strip.setPixelColor(loc + 24, color);
  }

  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 9, color);
    strip.setPixelColor(loc + 25, color);
  }

  if(col > -3 && col < 6) { 
    strip.setPixelColor(loc + 10, color);
    strip.setPixelColor(loc + 26, color);
  }
}
void drawPointed(uint32_t color, int row, int col) {
  /*
   *--- 
   *
   *---   
   *
   */

   
 int loc = row * 8 + col;
  
  if(col > -2 && col < 7) {
    strip.setPixelColor(loc + 9, color);
    strip.setPixelColor(loc + 25, color);
  }
}
void drawPlus(uint32_t color, int row, int col) {
  /*
   * - 
   *---
   * -   
   *
   */
 int loc = row * 8 + col;
   
  if(col > -1 && col < 8) {
   strip.setPixelColor(loc + 16, color); 
  }

  if(col > -2 && col < 7) {
   strip.setPixelColor(loc + 9, color);
   strip.setPixelColor(loc + 17, color); 
   strip.setPixelColor(loc + 25, color);
  }
  
  if(col > -3 && col < 6) { 
   strip.setPixelColor(loc + 18, color); 
  }
}
void drawMinus(uint32_t color, int row, int col) {
  /*
   * - 
   *---
   * -   
   *
   */
 int loc = row * 8 + col;
   
  if(col > -1 && col < 8) {
   strip.setPixelColor(loc + 16, color); 
  }

  if(col > -2 && col < 7) {
   strip.setPixelColor(loc + 17, color); 
  }
  
  if(col > -3 && col < 6) { 
   strip.setPixelColor(loc + 18, color); 
  }
}
/*
BLYNK_WRITE(V7) {
  int btnValue = param.asInt();

  if(btnValue == HIGH) {
    if(password[currentIndex] == led) {
      currentIndex++;
    } else {
      currentIndex = 0;
    }

    if(currentIndex == 4) {
      currentIndex = 0;
      Blynk.email("Match", "You entered the right password!!");
    }

    Blynk.virtualWrite(V7, LOW);
  }
}

BLYNK_WRITE(V6) {
  if(option != 2) {
    return;
  }
  
  int light = param.asInt();

  if(led == D1) {
    led1Brightness = light;
  } else if(led == D2) {
    led2Brightness = light;
  } else if(led == D3) {
    led3Brightness = light;
  } else if(led == D4){
    led4Brightness = light;
  } else {
    light == -1;
  }

  if(light > -1){
    Blynk.virtualWrite(virtualLed, light);
    analogWrite(led, light);
  }
}

BLYNK_WRITE(V1) {
  option = param.asInt();
}

}*/
